@@ -39,10 +39,10 @@ final class WP_Customize_Nav_Menus {
 	 *
 	 * @since 4.3.0
 	 *
-	 * @param object $manager An instance of the WP_Customize_Manager class.
+	 * @param WP_Customize_Manager $manager Customizer bootstrap instance.
 	 */
 	public function __construct( $manager ) {
-		$this->manager = $manager;
+		$this->manager                     = $manager;
 		$this->original_nav_menu_locations = get_nav_menu_locations();
 
 		// See https://github.com/xwp/wp-customize-snapshots/blob/962586659688a5b1fd9ae93618b7ce2d4e7a421c/php/class-customize-snapshot-manager.php#L469-L499
@@ -75,8 +75,8 @@ final class WP_Customize_Nav_Menus {
 	 *
 	 * @since 4.5.0
 	 *
-	 * @param array $nonces Array of nonces.
-	 * @return array $nonces Modified array of nonces.
+	 * @param string[] $nonces Array of nonces.
+	 * @return string[] Modified array of nonces.
 	 */
 	public function filter_nonces( $nonces ) {
 		$nonces['customize-menus'] = wp_create_nonce( 'customize-menus' );
@@ -95,15 +95,15 @@ final class WP_Customize_Nav_Menus {
 			wp_die( -1 );
 		}
 
-		$all_items = array();
+		$all_items  = array();
 		$item_types = array();
 		if ( isset( $_POST['item_types'] ) && is_array( $_POST['item_types'] ) ) {
 			$item_types = wp_unslash( $_POST['item_types'] );
 		} elseif ( isset( $_POST['type'] ) && isset( $_POST['object'] ) ) { // Back compat.
 			$item_types[] = array(
-				'type' => wp_unslash( $_POST['type'] ),
+				'type'   => wp_unslash( $_POST['type'] ),
 				'object' => wp_unslash( $_POST['object'] ),
-				'page' => empty( $_POST['page'] ) ? 0 : absint( $_POST['page'] ),
+				'page'   => empty( $_POST['page'] ) ? 0 : absint( $_POST['page'] ),
 			);
 		} else {
 			wp_send_json_error( 'nav_menus_missing_type_or_object_parameter' );
@@ -113,10 +113,10 @@ final class WP_Customize_Nav_Menus {
 			if ( empty( $item_type['type'] ) || empty( $item_type['object'] ) ) {
 				wp_send_json_error( 'nav_menus_missing_type_or_object_parameter' );
 			}
-			$type = sanitize_key( $item_type['type'] );
+			$type   = sanitize_key( $item_type['type'] );
 			$object = sanitize_key( $item_type['object'] );
-			$page = empty( $item_type['page'] ) ? 0 : absint( $item_type['page'] );
-			$items = $this->load_available_items_query( $type, $object, $page );
+			$page   = empty( $item_type['page'] ) ? 0 : absint( $item_type['page'] );
+			$items  = $this->load_available_items_query( $type, $object, $page );
 			if ( is_wp_error( $items ) ) {
 				wp_send_json_error( $items->get_error_code() );
 			}
@@ -135,7 +135,7 @@ final class WP_Customize_Nav_Menus {
 	 *                         'post_type' and 'taxonomy'. Default is 'post_type'.
 	 * @param string $object Optional. Accepts any registered taxonomy or post type name. Default is 'page'.
 	 * @param int    $page   Optional. The page number used to generate the query offset. Default is '0'.
-	 * @return WP_Error|array Returns either a WP_Error object or an array of menu items.
+	 * @return array|WP_Error An array of menu items on success, a WP_Error object on failure.
 	 */
 	public function load_available_items_query( $type = 'post_type', $object = 'page', $page = 0 ) {
 		$items = array();
@@ -146,16 +146,48 @@ final class WP_Customize_Nav_Menus {
 				return new WP_Error( 'nav_menus_invalid_post_type' );
 			}
 
+			/*
+			 * If we're dealing with pages, let's prioritize the Front Page,
+			 * Posts Page and Privacy Policy Page at the top of the list.
+			 */
+			$important_pages   = array();
+			$suppress_page_ids = array();
 			if ( 0 === $page && 'page' === $object ) {
-				// Add "Home" link. Treat as a page, but switch to custom on add.
-				$items[] = array(
-					'id'         => 'home',
-					'title'      => _x( 'Home', 'nav menu home label' ),
-					'type'       => 'custom',
-					'type_label' => __( 'Custom Link' ),
-					'object'     => '',
-					'url'        => home_url(),
-				);
+				// Insert Front Page or custom "Home" link.
+				$front_page = 'page' === get_option( 'show_on_front' ) ? (int) get_option( 'page_on_front' ) : 0;
+				if ( ! empty( $front_page ) ) {
+					$front_page_obj      = get_post( $front_page );
+					$important_pages[]   = $front_page_obj;
+					$suppress_page_ids[] = $front_page_obj->ID;
+				} else {
+					// Add "Home" link. Treat as a page, but switch to custom on add.
+					$items[] = array(
+						'id'         => 'home',
+						'title'      => _x( 'Home', 'nav menu home label' ),
+						'type'       => 'custom',
+						'type_label' => __( 'Custom Link' ),
+						'object'     => '',
+						'url'        => home_url(),
+					);
+				}
+
+				// Insert Posts Page.
+				$posts_page = 'page' === get_option( 'show_on_front' ) ? (int) get_option( 'page_for_posts' ) : 0;
+				if ( ! empty( $posts_page ) ) {
+					$posts_page_obj      = get_post( $posts_page );
+					$important_pages[]   = $posts_page_obj;
+					$suppress_page_ids[] = $posts_page_obj->ID;
+				}
+
+				// Insert Privacy Policy Page.
+				$privacy_policy_page_id = (int) get_option( 'wp_page_for_privacy_policy' );
+				if ( ! empty( $privacy_policy_page_id ) ) {
+					$privacy_policy_page = get_post( $privacy_policy_page_id );
+					if ( $privacy_policy_page instanceof WP_Post && 'publish' === $privacy_policy_page->post_status ) {
+						$important_pages[]   = $privacy_policy_page;
+						$suppress_page_ids[] = $privacy_policy_page->ID;
+					}
+				}
 			} elseif ( 'post' !== $object && 0 === $page && $post_type->has_archive ) {
 				// Add a post type archive link.
 				$items[] = array(
@@ -179,43 +211,65 @@ final class WP_Customize_Nav_Menus {
 				}
 			}
 
-			$posts = array_merge( $posts, get_posts( array(
+			$args = array(
 				'numberposts' => 10,
 				'offset'      => 10 * $page,
 				'orderby'     => 'date',
 				'order'       => 'DESC',
 				'post_type'   => $object,
-			) ) );
+			);
+
+			// Add suppression array to arguments for get_posts.
+			if ( ! empty( $suppress_page_ids ) ) {
+				$args['post__not_in'] = $suppress_page_ids;
+			}
+
+			$posts = array_merge(
+				$posts,
+				$important_pages,
+				get_posts( $args )
+			);
 
 			foreach ( $posts as $post ) {
 				$post_title = $post->post_title;
 				if ( '' === $post_title ) {
-					/* translators: %d: ID of a post */
+					/* translators: %d: ID of a post. */
 					$post_title = sprintf( __( '#%d (no title)' ), $post->ID );
 				}
+
+				$post_type_label = get_post_type_object( $post->post_type )->labels->singular_name;
+				$post_states     = get_post_states( $post );
+				if ( ! empty( $post_states ) ) {
+					$post_type_label = implode( ',', $post_states );
+				}
+
 				$items[] = array(
 					'id'         => "post-{$post->ID}",
 					'title'      => html_entity_decode( $post_title, ENT_QUOTES, get_bloginfo( 'charset' ) ),
 					'type'       => 'post_type',
-					'type_label' => get_post_type_object( $post->post_type )->labels->singular_name,
+					'type_label' => $post_type_label,
 					'object'     => $post->post_type,
-					'object_id'  => intval( $post->ID ),
-					'url'        => get_permalink( intval( $post->ID ) ),
+					'object_id'  => (int) $post->ID,
+					'url'        => get_permalink( (int) $post->ID ),
 				);
 			}
 		} elseif ( 'taxonomy' === $type ) {
-			$terms = get_terms( $object, array(
-				'child_of'     => 0,
-				'exclude'      => '',
-				'hide_empty'   => false,
-				'hierarchical' => 1,
-				'include'      => '',
-				'number'       => 10,
-				'offset'       => 10 * $page,
-				'order'        => 'DESC',
-				'orderby'      => 'count',
-				'pad_counts'   => false,
-			) );
+			$terms = get_terms(
+				array(
+					'taxonomy'     => $object,
+					'child_of'     => 0,
+					'exclude'      => '',
+					'hide_empty'   => false,
+					'hierarchical' => 1,
+					'include'      => '',
+					'number'       => 10,
+					'offset'       => 10 * $page,
+					'order'        => 'DESC',
+					'orderby'      => 'count',
+					'pad_counts'   => false,
+				)
+			);
+
 			if ( is_wp_error( $terms ) ) {
 				return $terms;
 			}
@@ -227,8 +281,8 @@ final class WP_Customize_Nav_Menus {
 					'type'       => 'taxonomy',
 					'type_label' => get_taxonomy( $term->taxonomy )->labels->singular_name,
 					'object'     => $term->taxonomy,
-					'object_id'  => intval( $term->term_id ),
-					'url'        => get_term_link( intval( $term->term_id ), $term->taxonomy ),
+					'object_id'  => (int) $term->term_id,
+					'url'        => get_term_link( (int) $term->term_id, $term->taxonomy ),
 				);
 			}
 		}
@@ -269,8 +323,13 @@ final class WP_Customize_Nav_Menus {
 			$p = 1;
 		}
 
-		$s = sanitize_text_field( wp_unslash( $_POST['search'] ) );
-		$items = $this->search_available_items_query( array( 'pagenum' => $p, 's' => $s ) );
+		$s     = sanitize_text_field( wp_unslash( $_POST['search'] ) );
+		$items = $this->search_available_items_query(
+			array(
+				'pagenum' => $p,
+				's'       => $s,
+			)
+		);
 
 		if ( empty( $items ) ) {
 			wp_send_json_error( array( 'message' => __( 'No results found.' ) ) );
@@ -293,7 +352,7 @@ final class WP_Customize_Nav_Menus {
 		$items = array();
 
 		$post_type_objects = get_post_types( array( 'show_in_nav_menus' => true ), 'objects' );
-		$query = array(
+		$query             = array(
 			'post_type'              => array_keys( $post_type_objects ),
 			'suppress_filters'       => true,
 			'update_post_term_cache' => false,
@@ -313,47 +372,60 @@ final class WP_Customize_Nav_Menus {
 
 		// Prepend list of posts with nav_menus_created_posts search results on first page.
 		$nav_menus_created_posts_setting = $this->manager->get_setting( 'nav_menus_created_posts' );
-		if ( 1 === $args['pagenum'] && $nav_menus_created_posts_setting && count( $nav_menus_created_posts_setting ) > 0 ) {
-			$stub_post_query = new WP_Query( array_merge(
-				$query,
-				array(
-					'post_status' => 'auto-draft',
-					'post__in' => $nav_menus_created_posts_setting->value(),
-					'posts_per_page' => -1,
+		if ( 1 === $args['pagenum'] && $nav_menus_created_posts_setting && count( $nav_menus_created_posts_setting->value() ) > 0 ) {
+			$stub_post_query = new WP_Query(
+				array_merge(
+					$query,
+					array(
+						'post_status'    => 'auto-draft',
+						'post__in'       => $nav_menus_created_posts_setting->value(),
+						'posts_per_page' => -1,
+					)
 				)
-			) );
-			$posts = array_merge( $posts, $stub_post_query->posts );
+			);
+			$posts           = array_merge( $posts, $stub_post_query->posts );
 		}
 
 		// Query posts.
 		$get_posts = new WP_Query( $query );
-		$posts = array_merge( $posts, $get_posts->posts );
+		$posts     = array_merge( $posts, $get_posts->posts );
 
 		// Create items for posts.
 		foreach ( $posts as $post ) {
 			$post_title = $post->post_title;
 			if ( '' === $post_title ) {
-				/* translators: %d: ID of a post */
+				/* translators: %d: ID of a post. */
 				$post_title = sprintf( __( '#%d (no title)' ), $post->ID );
 			}
+
+			$post_type_label = $post_type_objects[ $post->post_type ]->labels->singular_name;
+			$post_states     = get_post_states( $post );
+			if ( ! empty( $post_states ) ) {
+				$post_type_label = implode( ',', $post_states );
+			}
+
 			$items[] = array(
 				'id'         => 'post-' . $post->ID,
 				'title'      => html_entity_decode( $post_title, ENT_QUOTES, get_bloginfo( 'charset' ) ),
 				'type'       => 'post_type',
-				'type_label' => $post_type_objects[ $post->post_type ]->labels->singular_name,
+				'type_label' => $post_type_label,
 				'object'     => $post->post_type,
-				'object_id'  => intval( $post->ID ),
-				'url'        => get_permalink( intval( $post->ID ) ),
+				'object_id'  => (int) $post->ID,
+				'url'        => get_permalink( (int) $post->ID ),
 			);
 		}
 
 		// Query taxonomy terms.
 		$taxonomies = get_taxonomies( array( 'show_in_nav_menus' => true ), 'names' );
-		$terms = get_terms( $taxonomies, array(
-			'name__like' => $args['s'],
-			'number'     => 20,
-			'offset'     => 20 * ($args['pagenum'] - 1),
-		) );
+		$terms      = get_terms(
+			array(
+				'taxonomies' => $taxonomies,
+				'name__like' => $args['s'],
+				'number'     => 20,
+				'hide_empty' => false,
+				'offset'     => 20 * ( $args['pagenum'] - 1 ),
+			)
+		);
 
 		// Check if any taxonomies were found.
 		if ( ! empty( $terms ) ) {
@@ -364,12 +436,32 @@ final class WP_Customize_Nav_Menus {
 					'type'       => 'taxonomy',
 					'type_label' => get_taxonomy( $term->taxonomy )->labels->singular_name,
 					'object'     => $term->taxonomy,
-					'object_id'  => intval( $term->term_id ),
-					'url'        => get_term_link( intval( $term->term_id ), $term->taxonomy ),
+					'object_id'  => (int) $term->term_id,
+					'url'        => get_term_link( (int) $term->term_id, $term->taxonomy ),
 				);
 			}
 		}
 
+		// Add "Home" link if search term matches. Treat as a page, but switch to custom on add.
+		if ( isset( $args['s'] ) ) {
+			// Only insert custom "Home" link if there's no Front Page
+			$front_page = 'page' === get_option( 'show_on_front' ) ? (int) get_option( 'page_on_front' ) : 0;
+			if ( empty( $front_page ) ) {
+				$title   = _x( 'Home', 'nav menu home label' );
+				$matches = function_exists( 'mb_stripos' ) ? false !== mb_stripos( $title, $args['s'] ) : false !== stripos( $title, $args['s'] );
+				if ( $matches ) {
+					$items[] = array(
+						'id'         => 'home',
+						'title'      => $title,
+						'type'       => 'custom',
+						'type_label' => __( 'Custom Link' ),
+						'object'     => '',
+						'url'        => home_url(),
+					);
+				}
+			}
+		}
+
 		/**
 		 * Filters the available menu items during a search request.
 		 *
@@ -399,20 +491,20 @@ final class WP_Customize_Nav_Menus {
 		if ( 1 === $num_locations ) {
 			$locations_description = __( 'Your theme can display menus in one location.' );
 		} else {
-			/* translators: %s: number of menu locations */
+			/* translators: %s: Number of menu locations. */
 			$locations_description = sprintf( _n( 'Your theme can display menus in %s location.', 'Your theme can display menus in %s locations.', $num_locations ), number_format_i18n( $num_locations ) );
 		}
 
 		// Pass data to JS.
 		$settings = array(
-			'allMenus'             => wp_get_nav_menus(),
-			'itemTypes'            => $this->available_item_types(),
-			'l10n'                 => array(
+			'allMenus'                 => wp_get_nav_menus(),
+			'itemTypes'                => $this->available_item_types(),
+			'l10n'                     => array(
 				'untitled'               => _x( '(no label)', 'missing menu item navigation label' ),
 				'unnamed'                => _x( '(unnamed)', 'Missing menu name.' ),
 				'custom_label'           => __( 'Custom Link' ),
 				'page_label'             => get_post_type_object( 'page' )->labels->singular_name,
-				/* translators: %s:      menu location */
+				/* translators: %s: Menu location. */
 				'menuLocation'           => _x( '(Currently set to: %s)', 'menu' ),
 				'locationsTitle'         => 1 === $num_locations ? __( 'Menu Location' ) : __( 'Menu Locations' ),
 				'locationsDescription'   => $locations_description,
@@ -426,13 +518,15 @@ final class WP_Customize_Nav_Menus {
 				'movedDown'              => __( 'Menu item moved down' ),
 				'movedLeft'              => __( 'Menu item moved out of submenu' ),
 				'movedRight'             => __( 'Menu item is now a sub-item' ),
-				/* translators: &#9656; is the unicode right-pointing triangle, and %s is the section title in the Customizer */
+				/* translators: &#9656; is the unicode right-pointing triangle. %s: Section title in the Customizer. */
 				'customizingMenus'       => sprintf( __( 'Customizing &#9656; %s' ), esc_html( $this->manager->get_panel( 'nav_menus' )->title ) ),
-				/* translators: %s: title of menu item which is invalid */
+				/* translators: %s: Title of an invalid menu item. */
 				'invalidTitleTpl'        => __( '%s (Invalid)' ),
-				/* translators: %s: title of menu item in draft status */
+				/* translators: %s: Title of a menu item in draft status. */
 				'pendingTitleTpl'        => __( '%s (Pending)' ),
+				/* translators: %d: Number of menu items found. */
 				'itemsFound'             => __( 'Number of items found: %d' ),
+				/* translators: %d: Number of additional menu items found. */
 				'itemsFoundMore'         => __( 'Additional items found: %d' ),
 				'itemsLoadingMore'       => __( 'Loading more results... please wait.' ),
 				'reorderModeOn'          => __( 'Reorder mode enabled' ),
@@ -440,9 +534,9 @@ final class WP_Customize_Nav_Menus {
 				'reorderLabelOn'         => esc_attr__( 'Reorder menu items' ),
 				'reorderLabelOff'        => esc_attr__( 'Close reorder mode' ),
 			),
-			'settingTransport'     => 'postMessage',
-			'phpIntMax'            => PHP_INT_MAX,
-			'defaultSettingValues' => array(
+			'settingTransport'         => 'postMessage',
+			'phpIntMax'                => PHP_INT_MAX,
+			'defaultSettingValues'     => array(
 				'nav_menu'      => $temp_nav_menu_setting->default,
 				'nav_menu_item' => $temp_nav_menu_item_setting->default,
 			),
@@ -455,21 +549,21 @@ final class WP_Customize_Nav_Menus {
 		// This is copied from nav-menus.php, and it has an unfortunate object name of `menus`.
 		$nav_menus_l10n = array(
 			'oneThemeLocationNoMenus' => null,
-			'moveUp'       => __( 'Move up one' ),
-			'moveDown'     => __( 'Move down one' ),
-			'moveToTop'    => __( 'Move to the top' ),
-			/* translators: %s: previous item name */
-			'moveUnder'    => __( 'Move under %s' ),
-			/* translators: %s: previous item name */
-			'moveOutFrom'  => __( 'Move out from under %s' ),
-			/* translators: %s: previous item name */
-			'under'        => __( 'Under %s' ),
-			/* translators: %s: previous item name */
-			'outFrom'      => __( 'Out from under %s' ),
-			/* translators: 1: item name, 2: item position, 3: total number of items */
-			'menuFocus'    => __( '%1$s. Menu item %2$d of %3$d.' ),
-			/* translators: 1: item name, 2: item position, 3: parent item name */
-			'subMenuFocus' => __( '%1$s. Sub item number %2$d under %3$s.' ),
+			'moveUp'                  => __( 'Move up one' ),
+			'moveDown'                => __( 'Move down one' ),
+			'moveToTop'               => __( 'Move to the top' ),
+			/* translators: %s: Previous item name. */
+			'moveUnder'               => __( 'Move under %s' ),
+			/* translators: %s: Previous item name. */
+			'moveOutFrom'             => __( 'Move out from under %s' ),
+			/* translators: %s: Previous item name. */
+			'under'                   => __( 'Under %s' ),
+			/* translators: %s: Previous item name. */
+			'outFrom'                 => __( 'Out from under %s' ),
+			/* translators: 1: Item name, 2: Item position, 3: Total number of items. */
+			'menuFocus'               => __( '%1$s. Menu item %2$d of %3$d.' ),
+			/* translators: 1: Item name, 2: Item position, 3: Parent item name. */
+			'subMenuFocus'            => __( '%1$s. Sub item number %2$d under %3$s.' ),
 		);
 		wp_localize_script( 'nav-menu', 'menus', $nav_menus_l10n );
 	}
@@ -556,17 +650,30 @@ final class WP_Customize_Nav_Menus {
 		// Create a panel for Menus.
 		$description = '<p>' . __( 'This panel is used for managing navigation menus for content you have already published on your site. You can create menus and add items for existing content such as pages, posts, categories, tags, formats, or custom links.' ) . '</p>';
 		if ( current_theme_supports( 'widgets' ) ) {
-			/* translators: URL to the widgets panel of the customizer */
-			$description .= '<p>' . sprintf( __( 'Menus can be displayed in locations defined by your theme or in <a href="%s">widget areas</a> by adding a &#8220;Navigation Menu&#8221; widget.' ), "javascript:wp.customize.panel( 'widgets' ).focus();" ) . '</p>';
+			$description .= '<p>' . sprintf(
+				/* translators: %s: URL to the Widgets panel of the Customizer. */
+				__( 'Menus can be displayed in locations defined by your theme or in <a href="%s">widget areas</a> by adding a &#8220;Navigation Menu&#8221; widget.' ),
+				"javascript:wp.customize.panel( 'widgets' ).focus();"
+			) . '</p>';
 		} else {
 			$description .= '<p>' . __( 'Menus can be displayed in locations defined by your theme.' ) . '</p>';
 		}
-		$this->manager->add_panel( new WP_Customize_Nav_Menus_Panel( $this->manager, 'nav_menus', array(
-			'title'       => __( 'Menus' ),
-			'description' => $description,
-			'priority'    => 100,
-			// 'theme_supports' => 'menus|widgets', @todo allow multiple theme supports
-		) ) );
+
+		/*
+		 * Once multiple theme supports are allowed in WP_Customize_Panel,
+		 * this panel can be restricted to themes that support menus or widgets.
+		 */
+		$this->manager->add_panel(
+			new WP_Customize_Nav_Menus_Panel(
+				$this->manager,
+				'nav_menus',
+				array(
+					'title'       => __( 'Menus' ),
+					'description' => $description,
+					'priority'    => 100,
+				)
+			)
+		);
 		$menus = wp_get_nav_menus();
 
 		// Menu locations.
@@ -575,21 +682,24 @@ final class WP_Customize_Nav_Menus {
 		if ( 1 == $num_locations ) {
 			$description = '<p>' . __( 'Your theme can display menus in one location. Select which menu you would like to use.' ) . '</p>';
 		} else {
-			/* translators: %s: number of menu locations */
+			/* translators: %s: Number of menu locations. */
 			$description = '<p>' . sprintf( _n( 'Your theme can display menus in %s location. Select which menu you would like to use.', 'Your theme can display menus in %s locations. Select which menu appears in each location.', $num_locations ), number_format_i18n( $num_locations ) ) . '</p>';
 		}
 
 		if ( current_theme_supports( 'widgets' ) ) {
-			/* translators: URL to the widgets panel of the customizer */
+			/* translators: URL to the Widgets panel of the Customizer. */
 			$description .= '<p>' . sprintf( __( 'If your theme has widget areas, you can also add menus there. Visit the <a href="%s">Widgets panel</a> and add a &#8220;Navigation Menu widget&#8221; to display a menu in a sidebar or footer.' ), "javascript:wp.customize.panel( 'widgets' ).focus();" ) . '</p>';
 		}
 
-		$this->manager->add_section( 'menu_locations', array(
-			'title'       => 1 === $num_locations ? _x( 'View Location', 'menu locations' ) : _x( 'View All Locations', 'menu locations' ),
-			'panel'       => 'nav_menus',
-			'priority'    => 30,
-			'description' => $description,
-		) );
+		$this->manager->add_section(
+			'menu_locations',
+			array(
+				'title'       => 1 === $num_locations ? _x( 'View Location', 'menu locations' ) : _x( 'View All Locations', 'menu locations' ),
+				'panel'       => 'nav_menus',
+				'priority'    => 30,
+				'description' => $description,
+			)
+		);
 
 		$choices = array( '0' => __( '&mdash; Select &mdash;' ) );
 		foreach ( $menus as $menu ) {
@@ -618,13 +728,16 @@ final class WP_Customize_Nav_Menus {
 				remove_filter( "customize_sanitize_{$setting_id}", 'absint' );
 				add_filter( "customize_sanitize_{$setting_id}", array( $this, 'intval_base10' ) );
 			} else {
-				$this->manager->add_setting( $setting_id, array(
-					'sanitize_callback' => array( $this, 'intval_base10' ),
-					'theme_supports'    => 'menus',
-					'type'              => 'theme_mod',
-					'transport'         => 'postMessage',
-					'default'           => 0,
-				) );
+				$this->manager->add_setting(
+					$setting_id,
+					array(
+						'sanitize_callback' => array( $this, 'intval_base10' ),
+						'theme_supports'    => 'menus',
+						'type'              => 'theme_mod',
+						'transport'         => 'postMessage',
+						'default'           => 0,
+					)
+				);
 			}
 
 			// Override the assigned nav menu location if mapped during previewed theme switch.
@@ -632,12 +745,23 @@ final class WP_Customize_Nav_Menus {
 				$this->manager->set_post_value( $setting_id, $mapped_nav_menu_locations[ $location ] );
 			}
 
-			$this->manager->add_control( new WP_Customize_Nav_Menu_Location_Control( $this->manager, $setting_id, array(
-				'label'       => $description,
-				'location_id' => $location,
-				'section'     => 'menu_locations',
-				'choices'     => $choices,
-			) ) );
+			$this->manager->add_control(
+				new WP_Customize_Nav_Menu_Location_Control(
+					$this->manager,
+					$setting_id,
+					array(
+						'label'       => $description,
+						'location_id' => $location,
+						'section'     => 'menu_locations',
+						'choices'     => $choices,
+					)
+				)
+			);
+		}
+
+		// Used to denote post states for special pages.
+		if ( ! function_exists( 'get_post_states' ) ) {
+			require_once ABSPATH . 'wp-admin/includes/template.php';
 		}
 
 		// Register each menu as a Customizer section, and add each menu item to each menu.
@@ -646,16 +770,28 @@ final class WP_Customize_Nav_Menus {
 
 			// Create a section for each menu.
 			$section_id = 'nav_menu[' . $menu_id . ']';
-			$this->manager->add_section( new WP_Customize_Nav_Menu_Section( $this->manager, $section_id, array(
-				'title'     => html_entity_decode( $menu->name, ENT_QUOTES, get_bloginfo( 'charset' ) ),
-				'priority'  => 10,
-				'panel'     => 'nav_menus',
-			) ) );
+			$this->manager->add_section(
+				new WP_Customize_Nav_Menu_Section(
+					$this->manager,
+					$section_id,
+					array(
+						'title'    => html_entity_decode( $menu->name, ENT_QUOTES, get_bloginfo( 'charset' ) ),
+						'priority' => 10,
+						'panel'    => 'nav_menus',
+					)
+				)
+			);
 
 			$nav_menu_setting_id = 'nav_menu[' . $menu_id . ']';
-			$this->manager->add_setting( new WP_Customize_Nav_Menu_Setting( $this->manager, $nav_menu_setting_id, array(
-				'transport' => 'postMessage',
-			) ) );
+			$this->manager->add_setting(
+				new WP_Customize_Nav_Menu_Setting(
+					$this->manager,
+					$nav_menu_setting_id,
+					array(
+						'transport' => 'postMessage',
+					)
+				)
+			);
 
 			// Add the menu contents.
 			$menu_items = (array) wp_get_nav_menu_items( $menu_id );
@@ -671,36 +807,57 @@ final class WP_Customize_Nav_Menus {
 				}
 
 				$value['nav_menu_term_id'] = $menu_id;
-				$this->manager->add_setting( new WP_Customize_Nav_Menu_Item_Setting( $this->manager, $menu_item_setting_id, array(
-					'value'     => $value,
-					'transport' => 'postMessage',
-				) ) );
+				$this->manager->add_setting(
+					new WP_Customize_Nav_Menu_Item_Setting(
+						$this->manager,
+						$menu_item_setting_id,
+						array(
+							'value'     => $value,
+							'transport' => 'postMessage',
+						)
+					)
+				);
 
 				// Create a control for each menu item.
-				$this->manager->add_control( new WP_Customize_Nav_Menu_Item_Control( $this->manager, $menu_item_setting_id, array(
-					'label'    => $item->title,
-					'section'  => $section_id,
-					'priority' => 10 + $i,
-				) ) );
+				$this->manager->add_control(
+					new WP_Customize_Nav_Menu_Item_Control(
+						$this->manager,
+						$menu_item_setting_id,
+						array(
+							'label'    => $item->title,
+							'section'  => $section_id,
+							'priority' => 10 + $i,
+						)
+					)
+				);
 			}
 
 			// Note: other controls inside of this section get added dynamically in JS via the MenuSection.ready() function.
 		}
 
 		// Add the add-new-menu section and controls.
-		$this->manager->add_section( 'add_menu', array(
-			'type'     => 'new_menu',
-			'title'    => __( 'New Menu' ),
-			'panel'    => 'nav_menus',
-			'priority' => 20,
-		) );
+		$this->manager->add_section(
+			'add_menu',
+			array(
+				'type'     => 'new_menu',
+				'title'    => __( 'New Menu' ),
+				'panel'    => 'nav_menus',
+				'priority' => 20,
+			)
+		);
 
-		$this->manager->add_setting( new WP_Customize_Filter_Setting( $this->manager, 'nav_menus_created_posts', array(
-			'transport' => 'postMessage',
-			'type' => 'option', // To prevent theme prefix in changeset.
-			'default' => array(),
-			'sanitize_callback' => array( $this, 'sanitize_nav_menus_created_posts' ),
-		) ) );
+		$this->manager->add_setting(
+			new WP_Customize_Filter_Setting(
+				$this->manager,
+				'nav_menus_created_posts',
+				array(
+					'transport'         => 'postMessage',
+					'type'              => 'option', // To prevent theme prefix in changeset.
+					'default'           => array(),
+					'sanitize_callback' => array( $this, 'sanitize_nav_menus_created_posts' ),
+				)
+			)
+		);
 	}
 
 	/**
@@ -733,10 +890,10 @@ final class WP_Customize_Nav_Menus {
 		if ( $post_types ) {
 			foreach ( $post_types as $slug => $post_type ) {
 				$item_types[] = array(
-					'title'  => $post_type->labels->name,
+					'title'      => $post_type->labels->name,
 					'type_label' => $post_type->labels->singular_name,
-					'type' => 'post_type',
-					'object' => $post_type->name,
+					'type'       => 'post_type',
+					'object'     => $post_type->name,
 				);
 			}
 		}
@@ -748,10 +905,10 @@ final class WP_Customize_Nav_Menus {
 					continue;
 				}
 				$item_types[] = array(
-					'title' => $taxonomy->labels->name,
+					'title'      => $taxonomy->labels->name,
 					'type_label' => $taxonomy->labels->singular_name,
-					'type' => 'taxonomy',
-					'object' => $taxonomy->name,
+					'type'       => 'taxonomy',
+					'object'     => $taxonomy->name,
 				);
 			}
 		}
@@ -777,10 +934,10 @@ final class WP_Customize_Nav_Menus {
 	 * @param array $postarr {
 	 *     Post array. Note that post_status is overridden to be `auto-draft`.
 	 *
-	 *     @var string $post_title   Post title. Required.
-	 *     @var string $post_type    Post type. Required.
-	 *     @var string $post_name    Post name.
-	 *     @var string $post_content Post content.
+	 * @var string $post_title   Post title. Required.
+	 * @var string $post_type    Post type. Required.
+	 * @var string $post_name    Post name.
+	 * @var string $post_content Post content.
 	 * }
 	 * @return WP_Post|WP_Error Inserted auto-draft post object or error.
 	 */
@@ -789,10 +946,10 @@ final class WP_Customize_Nav_Menus {
 			return new WP_Error( 'unknown_post_type', __( 'Invalid post type.' ) );
 		}
 		if ( empty( $postarr['post_title'] ) ) {
-			return new WP_Error( 'empty_title', __( 'Empty title' ) );
+			return new WP_Error( 'empty_title', __( 'Empty title.' ) );
 		}
 		if ( ! empty( $postarr['post_status'] ) ) {
-			return new WP_Error( 'status_forbidden', __( 'Status is forbidden' ) );
+			return new WP_Error( 'status_forbidden', __( 'Status is forbidden.' ) );
 		}
 
 		/*
@@ -809,7 +966,7 @@ final class WP_Customize_Nav_Menus {
 			$postarr['meta_input'] = array();
 		}
 		$postarr['meta_input']['_customize_draft_post_name'] = $postarr['post_name'];
-		$postarr['meta_input']['_customize_changeset_uuid'] = $this->manager->changeset_uuid();
+		$postarr['meta_input']['_customize_changeset_uuid']  = $this->manager->changeset_uuid();
 		unset( $postarr['post_name'] );
 
 		add_filter( 'wp_insert_post_empty_content', '__return_false', 1000 );
@@ -841,7 +998,7 @@ final class WP_Customize_Nav_Menus {
 			wp_send_json_error( 'missing_params', 400 );
 		}
 
-		$params = wp_unslash( $_POST['params'] );
+		$params         = wp_unslash( $_POST['params'] );
 		$illegal_params = array_diff( array_keys( $params ), array( 'post_type', 'post_title' ) );
 		if ( ! empty( $illegal_params ) ) {
 			wp_send_json_error( 'illegal_params', 400 );
@@ -849,7 +1006,7 @@ final class WP_Customize_Nav_Menus {
 
 		$params = array_merge(
 			array(
-				'post_type' => '',
+				'post_type'  => '',
 				'post_title' => '',
 			),
 			$params
@@ -882,7 +1039,7 @@ final class WP_Customize_Nav_Menus {
 			}
 
 			$data = array(
-				/* translators: %1$s is the post type name and %2$s is the error message. */
+				/* translators: 1: Post type name, 2: Error message. */
 				'message' => sprintf( __( '%1$s could not be created: %2$s' ), $singular_name, $error->get_error_message() ),
 			);
 			wp_send_json_error( $data );
@@ -914,10 +1071,12 @@ final class WP_Customize_Nav_Menus {
 							<span class="menu-item-title<# if ( ! data.title ) { #> no-title<# } #>">{{ data.title || wp.customize.Menus.data.l10n.untitled }}</span>
 						</span>
 						<button type="button" class="button-link item-add">
-							<span class="screen-reader-text"><?php
-								/* translators: 1: Title of a menu item, 2: Type of a menu item */
+							<span class="screen-reader-text">
+							<?php
+								/* translators: 1: Title of a menu item, 2: Type of a menu item. */
 								printf( __( 'Add to menu: %1$s (%2$s)' ), '{{ data.title || wp.customize.Menus.data.l10n.untitled }}', '{{ data.type_label }}' );
-							?></span>
+							?>
+							</span>
 						</button>
 					</div>
 				</div>
@@ -969,11 +1128,11 @@ final class WP_Customize_Nav_Menus {
 				</button>
 			</h3>
 		</script>
-	<?php
+		<?php
 	}
 
 	/**
-	 * Print the html template used to render the add-menu-item frame.
+	 * Print the HTML template used to render the add-menu-item frame.
 	 *
 	 * @since 4.3.0
 	 */
@@ -987,7 +1146,7 @@ final class WP_Customize_Nav_Menus {
 				<h3>
 					<span class="customize-action">
 						<?php
-							/* translators: &#9656; is the unicode right-pointing triangle, and %s is the section title in the Customizer */
+							/* translators: &#9656; is the unicode right-pointing triangle. %s: Section title in the Customizer. */
 							printf( __( 'Customizing &#9656; %s' ), esc_html( $this->manager->get_panel( 'nav_menus' )->title ) );
 						?>
 					</span>
@@ -997,7 +1156,7 @@ final class WP_Customize_Nav_Menus {
 			<div id="available-menu-items-search" class="accordion-section cannot-expand">
 				<div class="accordion-section-title">
 					<label class="screen-reader-text" for="menu-items-search"><?php _e( 'Search Menu Items' ); ?></label>
-					<input type="text" id="menu-items-search" placeholder="<?php esc_attr_e( 'Search menu items&hellip;' ) ?>" aria-describedby="menu-items-search-desc" />
+					<input type="text" id="menu-items-search" placeholder="<?php esc_attr_e( 'Search menu items&hellip;' ); ?>" aria-describedby="menu-items-search-desc" />
 					<p class="screen-reader-text" id="menu-items-search-desc"><?php _e( 'The search results will be updated as you type.' ); ?></p>
 					<span class="spinner"></span>
 				</div>
@@ -1008,7 +1167,7 @@ final class WP_Customize_Nav_Menus {
 			<?php
 
 			// Ensure the page post type comes first in the list.
-			$item_types = $this->available_item_types();
+			$item_types     = $this->available_item_types();
 			$page_item_type = null;
 			foreach ( $item_types as $i => $item_type ) {
 				if ( isset( $item_type['object'] ) && 'page' === $item_type['object'] ) {
@@ -1027,7 +1186,7 @@ final class WP_Customize_Nav_Menus {
 			}
 			?>
 		</div><!-- #available-menu-items -->
-	<?php
+		<?php
 	}
 
 	/**
@@ -1049,9 +1208,12 @@ final class WP_Customize_Nav_Menus {
 				<span class="spinner"></span>
 				<span class="no-items"><?php _e( 'No items' ); ?></span>
 				<button type="button" class="button-link" aria-expanded="false">
-					<span class="screen-reader-text"><?php
-						/* translators: %s: Title of a section with menu items */
-						printf( __( 'Toggle section: %s' ), esc_html( $available_item_type['title'] ) ); ?></span>
+					<span class="screen-reader-text">
+					<?php
+						/* translators: %s: Title of a section with menu items. */
+						printf( __( 'Toggle section: %s' ), esc_html( $available_item_type['title'] ) );
+					?>
+						</span>
 					<span class="toggle-indicator" aria-hidden="true"></span>
 				</button>
 			</h4>
@@ -1093,7 +1255,7 @@ final class WP_Customize_Nav_Menus {
 				<input type="hidden" value="custom" id="custom-menu-item-type" name="menu-item[-1][menu-item-type]" />
 				<p id="menu-item-url-wrap" class="wp-clearfix">
 					<label class="howto" for="custom-menu-item-url"><?php _e( 'URL' ); ?></label>
-					<input id="custom-menu-item-url" name="menu-item[-1][menu-item-url]" type="text" class="code menu-item-textbox" value="http://">
+					<input id="custom-menu-item-url" name="menu-item[-1][menu-item-url]" type="text" class="code menu-item-textbox" placeholder="https://">
 				</p>
 				<p id="menu-item-name-wrap" class="wp-clearfix">
 					<label class="howto" for="custom-menu-item-name"><?php _e( 'Link Text' ); ?></label>
@@ -1183,7 +1345,7 @@ final class WP_Customize_Nav_Menus {
 	 * @since 4.7.0
 	 *
 	 * @param array $value Post IDs.
-	 * @returns array Post IDs.
+	 * @return array Post IDs.
 	 */
 	public function sanitize_nav_menus_created_posts( $value ) {
 		$post_ids = array();
@@ -1199,7 +1361,7 @@ final class WP_Customize_Nav_Menus {
 			if ( ! $post_type_obj ) {
 				continue;
 			}
-			if ( ! current_user_can( $post_type_obj->cap->publish_posts ) || ! current_user_can( $post_type_obj->cap->edit_post, $post_id ) ) {
+			if ( ! current_user_can( $post_type_obj->cap->publish_posts ) || ! current_user_can( 'edit_post', $post_id ) ) {
 				continue;
 			}
 			$post_ids[] = $post->ID;
@@ -1231,11 +1393,11 @@ final class WP_Customize_Nav_Menus {
 				}
 
 				$target_status = 'attachment' === get_post_type( $post_id ) ? 'inherit' : 'publish';
-				$args = array(
-					'ID' => $post_id,
+				$args          = array(
+					'ID'          => $post_id,
 					'post_status' => $target_status,
 				);
-				$post_name = get_post_meta( $post_id, '_customize_draft_post_name', true );
+				$post_name     = get_post_meta( $post_id, '_customize_draft_post_name', true );
 				if ( $post_name ) {
 					$args['post_name'] = $post_name;
 				}
@@ -1252,8 +1414,9 @@ final class WP_Customize_Nav_Menus {
 	 * Keep track of the arguments that are being passed to wp_nav_menu().
 	 *
 	 * @since 4.3.0
+	 *
 	 * @see wp_nav_menu()
-	 * @see WP_Customize_Widgets_Partial_Refresh::filter_dynamic_sidebar_params()
+	 * @see WP_Customize_Widgets::filter_dynamic_sidebar_params()
 	 *
 	 * @param array $args An array containing wp_nav_menu() arguments.
 	 * @return array Arguments.
@@ -1294,7 +1457,7 @@ final class WP_Customize_Nav_Menus {
 		// Empty out args which may not be JSON-serializable.
 		if ( ! $can_partial_refresh ) {
 			$exported_args['fallback_cb'] = '';
-			$exported_args['walker'] = '';
+			$exported_args['walker']      = '';
 		}
 
 		/*
@@ -1308,7 +1471,7 @@ final class WP_Customize_Nav_Menus {
 		ksort( $exported_args );
 		$exported_args['args_hmac'] = $this->hash_nav_menu_args( $exported_args );
 
-		$args['customize_preview_nav_menus_args'] = $exported_args;
+		$args['customize_preview_nav_menus_args']                            = $exported_args;
 		$this->preview_nav_menu_instance_args[ $exported_args['args_hmac'] ] = $exported_args;
 		return $args;
 	}
@@ -1328,9 +1491,9 @@ final class WP_Customize_Nav_Menus {
 	 */
 	public function filter_wp_nav_menu( $nav_menu_content, $args ) {
 		if ( isset( $args->customize_preview_nav_menus_args['can_partial_refresh'] ) && $args->customize_preview_nav_menus_args['can_partial_refresh'] ) {
-			$attributes = sprintf( ' data-customize-partial-id="%s"', esc_attr( 'nav_menu_instance[' . $args->customize_preview_nav_menus_args['args_hmac'] . ']' ) );
-			$attributes .= ' data-customize-partial-type="nav_menu_instance"';
-			$attributes .= sprintf( ' data-customize-partial-placement-context="%s"', esc_attr( wp_json_encode( $args->customize_preview_nav_menus_args ) ) );
+			$attributes       = sprintf( ' data-customize-partial-id="%s"', esc_attr( 'nav_menu_instance[' . $args->customize_preview_nav_menus_args['args_hmac'] . ']' ) );
+			$attributes      .= ' data-customize-partial-type="nav_menu_instance"';
+			$attributes      .= sprintf( ' data-customize-partial-placement-context="%s"', esc_attr( wp_json_encode( $args->customize_preview_nav_menus_args ) ) );
 			$nav_menu_content = preg_replace( '#^(<\w+)#', '$1 ' . str_replace( '\\', '\\\\', $attributes ), $nav_menu_content, 1 );
 		}
 		return $nav_menu_content;
